package cn.feast.coding.generator;

import java.util.Map;

public interface Generator {
    void generateModel(Map<String, Object> map);

    void generateDao(Map<String, Object> map);

    void generateDaoImpl(Map<String, Object> map);

    void generateController(Map<String, Object> map);

    void generateIService(Map<String, Object> map);

    void generateService(Map<String, Object> map);

    void generateJsp(Map<String, Object> map);

    void generateIndex(Map<String, Object> map);

    void generateUtils(Map<String, Object> map);

    void generateCommon(Map<String, Object> map);

    void generateDefined(Map<String, Object> map);
}
